package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.adapter.impl.netty.impl.TcpServer;
import com.zjyr.beidouservice.adapter.impl.netty.notify.SocketNotifyEvent;
import com.zjyr.beidouservice.common.impl.BeidouAdapterTypeName;
import com.zjyr.beidouservice.mapper.ControlDeviceMapper;
import com.zjyr.beidouservice.misc.HelperUtils;
import com.zjyr.beidouservice.pojo.entry.ControlDevice;
import com.zjyr.beidouservice.pojo.po.BeidouAdapterDevice;
import com.zjyr.beidouservice.pojo.vo.ControlAdapterSocketCtx;
import com.zjyr.beidouservice.pojo.vo.binary.ControllerStatus;
import com.zjyr.beidouservice.pojo.vo.binary.SensorStatusAck;
import com.zjyr.beidouservice.service.AdapterProtocolService;
import com.zjyr.beidouservice.service.BaidouAdapterService;
import com.zjyr.beidouservice.service.SensorDataService;
import io.netty.channel.Channel;
import jakarta.annotation.Nonnull;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationListener;
import org.springframework.stereotype.Service;

import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;

import static com.zjyr.beidouservice.common.impl.SocketEventName.SOCKET_EVT_CONNECT;

@Service
@Slf4j

public class BaidouSocketAdapterServiceImpl<T extends ApplicationEvent> implements BaidouAdapterService, ApplicationListener<T> {

    private final ConcurrentHashMap<Long, ControlAdapterSocketCtx> ctxMap = new ConcurrentHashMap<>();
    @Resource
    ControlDeviceMapper controlDeviceMapper;

    @Resource
    AdapterProtocolService adapterProtocolService;

    @Resource
    SensorDataService sensorDataService;

    @Resource
    private TcpServer tcpServer;

    @Override
    public void startBeidouAdapter() {
        tcpServer.start();
    }

    @Override
    public void sendCommond(Long adapterId, Object command) {
        ControlAdapterSocketCtx ctx = ctxMap.get(adapterId);

        if (ctx != null) {
            sendCommond(ctx.getChannel(), command);
        }
    }

    @Override
    public void sendCommond(Channel ch, Object command) {
        //if (command instanceof HeartProtocol heart) {
        tcpServer.sendData(ch, adapterProtocolService.createTransmissionProtocol(command));
        //}
    }

    @Override
    public void onApplicationEvent(@Nonnull T evtContent) {
        if (evtContent instanceof SocketNotifyEvent event) {
            InetSocketAddress sa = (InetSocketAddress) event.getCtxChannel().remoteAddress();
            switch (event.getSocketEvent()) {
                case SOCKET_EVT_CONNECT, SOCKET_EVT_IDLE_TIMEOUT -> {
                    sendCommond(event.getCtxChannel(), adapterProtocolService.createHeartProtocol());

                    if (event.getSocketEvent() == SOCKET_EVT_CONNECT) {
                        BeidouAdapterDevice a = BeidouAdapterDevice.builder()
                                                                   .adapterType(BeidouAdapterTypeName.ADAPTER_SOCKET_TCP)
                                                                   .adapterAddr(sa.getAddress().getHostAddress())
                                                                   .build();
                        addNewBeidouAdapter(a);

                        ControlAdapterSocketCtx sockCtx = new ControlAdapterSocketCtx(a.getId(), event.getCtxChannel());
                        ctxMap.put(a.getId(), sockCtx);
                    }
                }

                case SOCKET_EVT_DISCONNECT -> {
                    for (ConcurrentHashMap.Entry<Long, ControlAdapterSocketCtx> entry : ctxMap.entrySet()) {
                        if (entry.getValue().getChannel().id() == event.getCtxChannel().id()) {
                            ctxMap.remove(entry.getKey());
                        }
                    }
                }

                case SOCKET_EVT_RECV -> {
                    if (event.getEvtMessage() instanceof ControllerStatus status) {
                        log.info("+++{}, {}, {}",
                                 status.getWirelessStatus(),
                                 status.getTelphoneStatus(),
                                 HelperUtils.bytesToHexString(status.getBeidouSignalStrength()));
                        BeidouAdapterDevice a = BeidouAdapterDevice.builder()
                                                                   .adapterType(BeidouAdapterTypeName.ADAPTER_SOCKET_TCP)
                                                                   .adapterAddr(sa.getAddress().getHostAddress())
                                                                   .sendEnabled(status.getSendStatus().getValue())
                                                                   .beidouStatus(status.getBeidouStatus().getValue())
                                                                   .beidouChannelStatus(HelperUtils.bytesToHexString(status.getBeidouFreq()))
                                                                   .beidouSignalStrength(HelperUtils.bytesToHexString(status.getBeidouSignalStrength()))
                                                                   .wirelessStatus(status.getWirelessStatus().getValue())
                                                                   .telphoneStatus(status.getTelphoneStatus().getValue())
                                                                   .build();
                        addNewBeidouAdapter(a);
                    } /*else if (event.getEvtMessage() instanceof SensorCotrolAck ack) {
                        log.info("+++{}, {}, {}, {}, {}",
                                 ack.getSendStatus(),
                                 ack.getBeidouStatus(),
                                 ack.getWirelessStatus(),
                                 ack.getTelphoneStatus(),
                                 HelperUtils.bytesToHexString(ack.getBeidouFreq()));
                    }*/ else if (event.getEvtMessage() instanceof SensorStatusAck ack) {
                        log.info("+++Total Sensors: {}", ack.getSensorStatus().size());
                        sensorDataService.addNewSensor(ack.getSensorStatus());
                    }
                }
            }
        }

    }

    @Override
    public List<BeidouAdapterDevice> getAllAdapter() {
        List<ControlDevice> controlDevices = controlDeviceMapper.selectAll();
        if (!controlDevices.isEmpty()) {
            List<BeidouAdapterDevice> adapterDevices = new ArrayList<>();

            for (ControlDevice c : controlDevices) {
                adapterDevices.add(BeidouAdapterDevice.builder()
                                                      .id(c.getId())
                                                      .adapterType(c.getDeviceType())
                                                      .adapterAddr(c.getDeviceAddr())
                                                      .sendEnabled(Optional.ofNullable(c.getSendEnabled()).orElse(0))
                                                      .beidouStatus(Optional.ofNullable(c.getBeidouStatus()).orElse(0))
                                                      .beidouChannelStatus(Optional.ofNullable(c.getBeidouChannelStatus()).orElse(""))
                                                      .beidouSignalStrength(Optional.ofNullable(c.getBeidouSignalStrength()).orElse(""))
                                                      .wirelessStatus(Optional.ofNullable(c.getWirelessStatus()).orElse(0))
                                                      .telphoneStatus(Optional.ofNullable(c.getTelphoneStatus()).orElse(0))
                                                      .build());
            }

            return adapterDevices;
        }

        return new ArrayList<>();
    }

    @Override
    public void addNewBeidouAdapter(BeidouAdapterDevice adapter) {
        ControlDevice device = ControlDevice.builder()
                                            .id(adapter.getId())
                                            .deviceType(adapter.getAdapterType())
                                            .deviceAddr(adapter.getAdapterAddr())
                                            .beidouStatus(adapter.getBeidouStatus())
                                            .sendEnabled(adapter.getSendEnabled())
                                            .beidouChannelStatus(adapter.getBeidouChannelStatus())
                                            .beidouSignalStrength(adapter.getBeidouSignalStrength())
                                            .wirelessStatus(adapter.getWirelessStatus())
                                            .telphoneStatus(adapter.getTelphoneStatus())
                                            .build();
        BeidouAdapterDevice dev = getAdapterByAddr(adapter.getAdapterAddr());
        if (dev == null) {
            controlDeviceMapper.addControlDevice(device);
        } else {
            device.setId(dev.getId());
            adapter.setId(dev.getId());
            controlDeviceMapper.upgradeControlDevice(device);
        }
    }

    @Override
    public BeidouAdapterDevice getAdapterByAddr(String addr) {
        for (BeidouAdapterDevice v : getAllAdapter()) {
            if (addr.equals(v.getAdapterAddr())) {
                return v;
            }
        }
        return null;
    }
}
